<?php /*
 * Plugin Name: Alfasearch Bundle
 * Plugin URI:  ###
 * Description: ACF-based Gutenberg Blocks for Wordpress
 * Version: 1.4.1
 * Author: Alfasearch WebDev team
 * Author URI: 
 * License: GPLv2 or later
 * License URI: https://www.gnu.org/licenses/gpl-2.0.html
 *
 * Text Domain: alfab
 * Domain Path: /alfab
 *
 * Network: true
 */

 defined( 'ABSPATH' ) || exit;

 final class AlfaBundle {

    protected static $instance = null;

    /**
	 * Holds various class instances.
	 *
	 * @var array
	 */
	private $container = [];

	/**
	 * Holds various routes.
	 *
	 * @var array
	 */
	private $routes = [];

	/**
	 * Hold install error messages.
	 *
	 * @var bool
	 */
	private $messages = [];

    /**
	 * Retrieve main AlfaBundle instance.
	 *
	 * Ensure only one instance is loaded or can be loaded.
	 *
	 * @see ab()
	 * @return AlfaBundle
	 */

	public static function get() {
		if ( is_null( self::$instance ) && ! ( self::$instance instanceof AlfaBundle ) ) {
			self::$instance = new AlfaBundle();
			self::$instance->setup();
		}

		return self::$instance;
	}

    /**
	 * Instantiate the plugin.
	 */
	private function setup() {
		
        
        $this->define_constants();
        $this->include_files();
		$this->init_actions();

	}

    private function init_actions() {

        // Booting.

		/* Back-end */
		
		add_action( 'acf/init', [ $this, 'acf_add_options_page' ], 5 );
		add_action( 'acf/init', [ $this, 'register_blocks' ], 10 );
		add_action( 'acf/init', [ $this, 'acf_import_json' ], 20 );
		add_action( 'rest_api_init', [ $this, 'init_rest_api' ] );	

		/* Front-end */

		add_filter( 'the_content', [ $this, 'wrap_content' ], 10, 1 );		

    }


	
    public function acf_add_options_page(){

        $args = array(
            'page_title' => 'Версии и ссылки',
            'menu_slug' => 'acf-options',
            'icon_url' => 'dashicons-plugins-checked',
            'updated_message'	=> "Значения обновлены"         
        );
        
        acf_add_options_page( $args );

    }

	public function wrap_content($content){

		$result = str_replace(
			array( '<h2' ), 
			array( '</section><section class="added-dynamic"><h2' ), 
		
		$content);

		$section__counter = 1;
		$header__counter = 1;

		$result = preg_replace_callback('|<section(.*)>|Uis', function($matches) {

			global $section__counter;
			$section__counter++;
	
			return '<section class="added-dynamic" id="section__'. $section__counter .'">';
		
		}, $result);

		$content = preg_replace_callback('|<h2(.*)</h2>|Uis', function($matches) use (&$headers) {

			$match = trim(strip_tags($matches[1]));
            $match = strstr($match, '>');
            $match = str_replace('>', '', $match);
            $heading = strtolower($match);
            $heading = str_replace(
                array(' ', ',', '!', '?', ':', '.','&nbsp;','(',')','¿'), 
                array('-', '','','','','','','','',''),
                $heading
            );
            
            $dash = (is_numeric($match[0]) ? '_' : '' );

			return '<h2 id="'.$dash.$heading.'">' . $match . '</h2>';
		
		}, $result);

		$content = str_replace('</section><section class="added-dynamic" id="section__1">', '<section class="added-dynamic" id="section__1">', $content );
		$content .= '<div class="emptiness"></div>';
		$content = str_replace('<div class="emptiness"></div>', '<div class="emptiness"></div></section>', $content );
		$content = str_replace('frameborder="0"', '', $content);
		return $content;

	}

	public function get_post_by_name($post_name, $output = OBJECT) {
		global $wpdb;
			$post = $wpdb->get_var( $wpdb->prepare( "SELECT ID FROM $wpdb->posts WHERE post_name = %s", $post_name ));
			if ( $post )
				return get_post($post, $output);
	
		return null;
	}


	public function acf_import_json(){

		if (function_exists('acf_import_field_group')) {
			
			$acf_json_data = AB_PLUGIN_DIR . 'acf.json';

			$custom_fields = $acf_json_data ? json_decode(file_get_contents($acf_json_data), true) : array();
	
			foreach ($custom_fields as $custom_field) {
				if (!acf_get_field_group($custom_field['key'])){
					acf_import_field_group($custom_field);
				}	
			}

		}

	}

	public function register_blocks(){

		foreach( $this->container['blocks'] as $block ){

			$name = strtolower($block);

			$settings = array(
				'name'              => 'alfa-'. $name,
				'title'             => 'Alfa '. $block,
				'mode'				=> 'edit',
				'description'       => 'A custom '. $block .' block.',
				'render_template'   => AB_PLUGIN_DIR . 'blocks/'. $name .'/'. $block .'.php',
				'category'          => 'formatting',
			);

			if ( file_exists( AB_PLUGIN_DIR . 'blocks/'. $name .'/icon.svg' ) )

				$settings['icon'] = file_get_contents( AB_PLUGIN_DIR . 'blocks/'. $name .'/icon.svg' );

			if ( file_exists( AB_PLUGIN_DIR . 'blocks/'. $name .'/'.$name.'.css' ) )

				$settings['enqueue_style'] =  '/wp-content/plugins/alfabundle/blocks/'. $name .'/'.$name.'.css';

			if ( file_exists( AB_PLUGIN_DIR .'blocks/'. $name .'/'.$name.'.js' ) && is_admin() )

				$settings['enqueue_script'] = '/wp-content/plugins/alfabundle/blocks/'. $name .'/'.$name.'.js';

			acf_register_block_type($settings);

		}

	}


    private function include_files() {

		// BLOCKS

		$this->container['blocks'] = [];

        foreach ( glob( AB_PLUGIN_DIR . 'blocks/*/*.php' ) as $file ) {
            
			if ( file_exists( $file ) ) {

				$class = pathinfo($file, PATHINFO_FILENAME);

				array_push($this->container['blocks'], $class);
			}
        
        }

	}


    /**
	 * Define the plugin constants.
	 */
	private function define_constants() {
		// Plugin DIR, with trailing slash:
        if( !defined('AB_PLUGIN_DIR') ) {
            define('AB_PLUGIN_DIR', plugin_dir_path(__FILE__));
        }
	}

	/**
	 * Load the REST API endpoints and callbacks.
	 */
	public function init_rest_api() {

		foreach ( glob( AB_PLUGIN_DIR . 'routes/*.php' ) as $file ) {
            
			if ( file_exists( $file ) ) {

				include_once $file;

				$route = pathinfo($file, PATHINFO_FILENAME);

				array_push($this->routes, $route);

			}
        
        }

	}


 }

	/**
	 * Returns the main instance of AlfaBundle to prevent the need to use globals.
	 *
	 * @return AlfaBundle
	 */
	function ab() {
		return AlfaBundle::get();
	}
	

	// Start it.
	ab();